// Wild Magic Source Code
// David Eberly
// http://www.geometrictools.com
// Copyright (c) 1998-2008
//
// This library is free software; you can redistribute it and/or modify it
// under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation; either version 2.1 of the License, or (at
// your option) any later version.  The license is available for reading at
// either of the locations:
//     http://www.gnu.org/copyleft/lgpl.html
//     http://www.geometrictools.com/License/WildMagicLicense.pdf
//
// Version: 4.0.0 (2006/06/28)

#include "ScenePrinter.h"

WM4_CONSOLE_APPLICATION(ScenePrinter);

//----------------------------------------------------------------------------
void ScenePrinter::Usage ()
{
    std::cout << std::endl;
    std::cout << "To print a scene graph in ASCII format, use" << std::endl;
    std::cout << "    ScenePrinter myfile.wmof myfile.txt" << std::endl;
}
//----------------------------------------------------------------------------
int ScenePrinter::Main (int iQuantity, char** aacArgument)
{
    if (iQuantity != 3)
    {
        Usage();
        return -1;
    }

    // get files
    const char* acInFile = aacArgument[1];
    const char* acOutFile = aacArgument[2];

    Stream* pkStream = new Stream;
    bool bLoaded = pkStream->Load(acInFile);
    if (!bLoaded)
    {
       return -2;
    }

    for (int i = 0; i < pkStream->GetObjectCount(); i++)
    {
        Spatial* pkSpatial = DynamicCast<Spatial>(pkStream->GetObjectAt(i));
        if (pkSpatial)
        {
            pkSpatial->UpdateGS();
            pkSpatial->UpdateRS();
        }
    }

    bool bSaved = pkStream->SaveText(acOutFile);
    if (!bSaved)
    {
        delete pkStream;
        return -3;
    }

    delete pkStream;
    return 0;
}
//----------------------------------------------------------------------------
